#!/usr/bin/env python
#
# getops.py : get SAP IoT Connect 365 list of known operators
#
import json
import requests
import argparse
import sys
import dialogs
import configparser
from requests.exceptions import ProxyError

global auth_body, auth_token, useProxies
global TMOUS, STATUS200

config = configparser.ConfigParser()
config.read('./resources/config.ini')

demo_user = config['API']['user']
demo_password = config['API']['password']
base_url = config['API']['base_url']
auth_url = base_url + "/authenticate"
endpoint_url = base_url + "/endpoint"
getoperator_url = base_url + "/operator"
STATUS200=200
auth_token=None
verbose = 0
useProxies = 0

def enableProxies() :
    global useProxies
    useProxies = 1 
    return

def disableProxies() :
    global useProxies
    useProxies = 0
    return

def getProxies() :
    global useProxies
    if useProxies :
        return {
            'http': 'http://bluecoat-proxy:8080',
            'https': 'http://proxy.phl.sap.corp:8080'
        }
    return {}

def getAuthToken():
    global verbose, auth_token
    auth_body = {"username": demo_user, "password": demo_password}
    try :                       # try SAP proxies first
        if (verbose) : print("auth 1st try")
        resp = requests.post(auth_url, json=auth_body, proxies=getProxies())
    except ProxyError as exc:                     # otherwise no proxies
        disableProxies()
        if (verbose) :
            print("Oops!",sys.exc_info()[0],"occurred.")
            print(exc)
            print("auth 2nd try")
        resp = requests.post(auth_url, json=auth_body, proxies=getProxies())
    if STATUS200 == resp.status_code :
        if (verbose) : print("authenticated ",resp) 
        auth_token = json.loads(resp.text)['auth_token']
    else :
        print("error**** in auth",resp)
        auth_token = None
    return

def getHeaders():
    global auth_token
    return {'Content-Type': 'application/json',
            'Authorization': 'Bearer ' +auth_token}

def getOps(f,l): # use filter function f on list l of operators
    match = list(filter(f,l))   # filtered list
    for o in match[:]:
        o['title'] = '{:5} {:5} {}'.format(
            o['id'],
            o['country']['country_code'],
            o['name_and_country'])
    return match

def getArgs() :
    parser = argparse.ArgumentParser(description=
        "List the operator identifiers available in the IoT Connect 365 " \
        "system by country name or country code. These identifiers can be"\
        "used by other utilities for example to manage endpoint blacklists."\
        "With no arguments, it takes name from console (like --name))")
    grp = parser.add_mutually_exclusive_group()
    grp.add_argument("-a","--all",
                     action="store_true",help="List all operators")
    grp.add_argument("-e","--endpoints",
                     action="store_true",help="List all endpoints")
    grp.add_argument("-n","--name",
                     help="name search within Country name or Operator name "\
                     "e.g. Austr")
    grp.add_argument("-c","--ccode",
                     help="Country code e.g. 49",type=int)
    parser.add_argument("-v","--verbose",
                        action="store_true")
    return parser.parse_args()

def modify_blacklist(addOp,opId,eId):
    global endpoint_url
    url = endpoint_url + "/" + str(eId) + "/operator_blacklist/" + str(opId)
    h=getHeaders()
    if addOp : return requests.put(url,headers=h,proxies=getProxies())
    else     : return requests.delete(url,headers=h,proxies=getProxies())

def add_blacklist(eId,opId):
    return modify_blacklist(1,opId,eId)

def del_blacklist(eId,opId):
    return modify_blacklist(0,opId,eId)

def get_blacklist(eId):
    h=getHeaders()
    resp = requests.get(endpoint_url + "/" + str(eId)
                        + "/operator_blacklist",
                        headers=h,
                        proxies=getProxies())
    blacklist=json.loads(resp.text)
    rlist=[]
    for b in blacklist[:] :
        rlist.append({'title':'{} {} / {}'.format(b['id'],
                                                 b['country']['name'],
                                                 b['name']),
                      'id':b['id']})
    return rlist


def getOperatorList(matchString) :
    resp = requests.get(getoperator_url,
                        headers=getHeaders(),
                        proxies=getProxies())
    if not(STATUS200 == resp.status_code) :
        print("Error in operator call:",resp.status_code,resp.text)
        quit()
    ops = json.loads(resp.text)
    #
    # now filter by supplied criteria and display
    #
    matchList = getOps(lambda e :
                       matchString.lower() in e['name_and_country'].lower(),
                       ops)
    return matchList

def processShowBlacklist(ept) :
    blist = get_blacklist(ept)
    act = dialogs.list_dialog("Blacklist for " + str(ept),blist)
    return

def processAddBlacklist(ept) :
    matchString = dialogs.text_dialog("Search operators for:","")
    matchList = getOperatorList(matchString)
    operator = dialogs.list_dialog("Add blacklist for " + str(ept),matchList)
    if (operator) :
        add_blacklist(ept,operator['id'])
    return

def processDelBlacklist(ept) :
    blist = get_blacklist(ept)
    operator = dialogs.list_dialog("Delete blacklist for " + str(ept),blist)
    if (operator) :
        del_blacklist(ept,operator['id'])
    return


#############################################################
# begin
# parse out the arguments
#
args = getArgs()
verbose = args.verbose
#
# Create Authorization Token
#
getAuthToken()
if not(auth_token) : quit()
#
#
#
resp = requests.get(endpoint_url + '?sort=-last_updated', # newest first
                    headers=getHeaders(),
                proxies=getProxies())
if not(STATUS200 == resp.status_code) :
    print("Error in endpoint call:",resp.status_code,resp.text)
    exit(2)
endpts = json.loads(resp.text)
if (verbose) : print("Endpoints found: {}".format(len(endpts)))
#if (len(endpts)>0):
#    print("{:>8} {:20.20} {:>16.16} {:>8}".format("Id",
#                                                "Name",
#                                                  "Last Updated",
#                                                  "ICCID"))
es=[]
for e in endpts[:]:
    es.append(
        {'title':'{:8} {:20.20} {:16.16} ...'.format(
            e['id'],
            e['name'],
            e['last_updated']),
#           e['sim']['iccid'][-5:]),
         'id':e['id']})
i = dialogs.list_dialog("Choose an endpoint",es);
ept=i['id']
#print(i,ept)

#resp = requests.get(endpoint_url + '/{:d}/stats'.format(ept),
#                    headers=getHeaders(),
#                    proxies=getProxies())
#if not(STATUS200 == resp.status_code) :
#    print("Error in endpoint stats call:",resp.status_code,resp.text)
#    exit(2)
#stats = json.loads(resp.text)
#print("current month data volume",
#      stats['current_month']['data']['volume'])

actions = [{'title':'add to blacklist','n':0},
           {'title':'delete from blacklist','n':1},
        {'title':'show blacklist','n':2}]
act=1
while (act) :
    act = dialogs.list_dialog("Choose an action",actions);
    if (not(act)) : break
    elif (act['n']==0) : processAddBlacklist(ept)
    elif (act['n']==1) : processDelBlacklist(ept)
    elif (act['n']==2) : processShowBlacklist(ept)
    else : exit(2)
    
quit()

resp = get_blacklist(ept)
blacklist = json.loads(resp.text)
if (len(blacklist)==0) : print ("No blacklist for ",ept)

ops = getOps()
if (ops) :
    for o in ops[:] :
        r = add_blacklist(ept,o['id'])
        
        
    #
    # Get list of all operators
    #
    resp = requests.get(getoperator_url,
                        headers=getHeaders(),
                        proxies=getProxies())
    if not(STATUS200 == resp.status_code) :
        print("Error in operator call:",resp.status_code,resp.text)
        quit()
        ops = json.loads(resp.text)
    #
    # now filter by supplied criteria and display
    #
    if   (args.ccode): # filter by country code
        match = getOps(lambda e :
                       int(e['country']['country_code']) == args.ccode,
                       ops)
    elif (args.all) : # don't filter 
        match = getOps(lambda e : 1, ops)
    else            : # filter by name (ignore case)
        if (args.name) : nameMatch = args.name 
        else :
            nameMatch = input("Enter search to match country/operator name " \
                              "(or just hit return for all)")
            match = getOps(lambda e :
                           nameMatch.lower() in e['name_and_country'].lower(),
                           ops)
            
            quit()
            
